from pathlib import Path
from typing import List

import datasets
from datasets import NamedSplit

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_DATASETNAME = "wikiann"

_LANGUAGES = ["ind", "jav", "min", "sun", "ace", "zlm", "map-bms", "mya", "tgl", "tha", "vie", "khm"]
_LOCAL = False
_CITATION = """\
@inproceedings{pan-etal-2017-cross,
    title = "Cross-lingual Name Tagging and Linking for 282 Languages",
    author = "Pan, Xiaoman  and
      Zhang, Boliang  and
      May, Jonathan  and
      Nothman, Joel  and
      Knight, Kevin  and
      Ji, Heng",
    booktitle = "Proceedings of the 55th Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)",
    month = jul,
    year = "2017",
    address = "Vancouver, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/P17-1178",
    doi = "10.18653/v1/P17-1178",
    pages = "1946--1958",
    abstract = "The ambitious goal of this work is to develop a cross-lingual name tagging and linking framework
    for 282 languages that exist in Wikipedia. Given a document in any of these languages, our framework is able
    to identify name mentions, assign a coarse-grained or fine-grained type to each mention, and link it to
    an English Knowledge Base (KB) if it is linkable. We achieve this goal by performing a series of
    new KB mining methods: generating {``}silver-standard{''} annotations by
    transferring annotations from English to other languages through cross-lingual links and KB properties,
    refining annotations through self-training and topic selection,
    deriving language-specific morphology features from anchor links, and mining word translation pairs from
    cross-lingual links. Both name tagging and linking results for 282 languages are promising
    on Wikipedia data and on-Wikipedia data.",
}
@inproceedings{rahimi-etal-2019-massively,
    title = "Massively Multilingual Transfer for {NER}",
    author = "Rahimi, Afshin  and
      Li, Yuan  and
      Cohn, Trevor",
    booktitle = "Proceedings of the 57th Annual Meeting of the Association for Computational Linguistics",
    month = jul,
    year = "2019",
    address = "Florence, Italy",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/P19-1015",
    pages = "151--164",
}
"""

_DESCRIPTION = """\
The wikiann dataset contains NER tags with labels from O (0), B-PER (1), I-PER (2), B-ORG (3), I-ORG (4), B-LOC (5), I-LOC (6). The Indonesian subset is used.
WikiANN (sometimes called PAN-X) is a multilingual named entity recognition dataset consisting of Wikipedia articles
 annotated with LOC (location), PER (person), and ORG (organisation)
 tags in the IOB2 format. This version corresponds to the balanced train, dev, and test splits of
  Rahimi et al. (2019), and uses the following subsets from the original WikiANN corpus

Language	WikiAnn	ISO 639-3
Indonesian	id	ind
Javanese	jv	jav
Minangkabau	min	min
Sundanese	su	sun
Acehnese	ace	ace
Malay	ms	zlm
Banyumasan	map-bms	map-bms
Myanmar my mya
Tagalog tl tgl
Thailand th tha
Vietnam vi vie
Khmer km khm


"""

_HOMEPAGE = "https://github.com/afshinrahimi/mmner"

_LICENSE = Licenses.APACHE_2_0.value

_URL = "https://s3.amazonaws.com/datasets.huggingface.co/wikiann/1.1.0/panx_dataset.zip"

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]

_SOURCE_VERSION = "1.1.0"
_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(lang, schema, version):
    if lang == "":
        raise ValueError(f"Invalid lang {lang}")

    if schema != "source" and schema != "seacrowd_seq_label":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="wikiann_{lang}_{schema}".format(lang=lang, schema=schema),
        version=datasets.Version(version),
        description="wikiann with {schema} schema for {lang} language".format(lang=lang, schema=schema),
        schema=schema,
        subset_id="wikiann",
    )


LANGUAGES_MAP = {"ind": "indonesian", "jav": "javanese", "min": "minangkabau", "sun": "sundanese", "ace": "acehnese", "zlm": "malay", "map_bms": "banyumasan", "mya": "myanmar", "tgl": "tagalog", "tha": "thailand", "vie": "vietnam", "khm": "khmer"}
LANG_CODES = {"ind": "id", "jav": "jv", "min": "min", "sun": "su", "ace": "ace", "zlm": "ms", "map_bms": "map-bms", "mya": "my", "tgl": "tl", "tha": "th","vie": "vi","khm": "km"}


class WikiAnnDataset(datasets.GeneratorBasedBuilder):
    """wikiann is an NER tagging dataset consisting of Wikipedia articles annotated with LOC, PER, and ORG tags
    for multiple Indonesian language. If the language is not specified, it loads the Indonesian subset."""

    label_classes = ["B-PER", "I-PER", "B-ORG", "I-ORG", "B-LOC", "I-LOC", "O"]

    BUILDER_CONFIGS = [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in LANGUAGES_MAP] + [seacrowd_config_constructor(lang, "seacrowd_seq_label", _SEACROWD_VERSION) for lang in LANGUAGES_MAP]

    DEFAULT_CONFIG_NAME = "wikiann_ind_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "tokens": [datasets.Value("string")], "ner_tag": [datasets.Value("string")]})
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def get_lang(self, name):
        return name.removesuffix("_source").removesuffix("_seacrowd_seq_label").removeprefix("wikiann_")

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        path = Path(dl_manager.download_and_extract(_URL))
        lang = LANG_CODES[self.get_lang(self.config.name)]
        wikiann_dl_dir = path / f"{lang}.tar.gz"
        return [
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"split": "dev", "filepath": dl_manager.iter_archive(wikiann_dl_dir)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"split": "test", "filepath": dl_manager.iter_archive(wikiann_dl_dir)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"split": "train", "filepath": dl_manager.iter_archive(wikiann_dl_dir)},
            ),
            datasets.SplitGenerator(
                name=NamedSplit("extra"),
                gen_kwargs={"split": "extra", "filepath": dl_manager.iter_archive(wikiann_dl_dir)},
            ),
        ]

    def _generate_examples(self, filepath: Path, split):
        """Based on https://github.com/huggingface/datasets/blob/main/datasets/wikiann/wikiann.py"""
        fps = filepath
        tokens = []
        ner_tags = []
        langs = []
        guid_index = 0
        for k, file in fps:
            if k == split:
                for line in file:
                    line = line.decode("utf-8")
                    if line == "" or line == "\n":
                        if tokens:
                            if self.config.schema == "source":
                                yield guid_index, {"index": str(guid_index), "tokens": tokens, "ner_tag": ner_tags}
                            elif self.config.schema == "seacrowd_seq_label":
                                yield guid_index, {"id": str(guid_index), "tokens": tokens, "labels": ner_tags}
                            else:
                                raise ValueError(f"Invalid config: {self.config.name}")
                            guid_index += 1
                            tokens = []
                            ner_tags = []
                            langs = []
                    else:
                        # wikiann data is tab separated
                        splits = line.split("\t")
                        # strip out en: prefix
                        langs.append(splits[0].split(":")[0])
                        tokens.append(":".join(splits[0].split(":")[1:]))
                        if len(splits) > 1:
                            ner_tags.append(splits[-1].replace("\n", ""))
                        else:
                            # examples have no label in test set
                            ner_tags.append("O")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@inproceedings{PhoNER_COVID19,
title     = {{COVID-19 Named Entity Recognition for Vietnamese}},
author    = {Thinh Hung Truong and Mai Hoang Dao and Dat Quoc Nguyen},
booktitle = {Proceedings of the 2021 Conference of the North American Chapter of the Association for Computational Linguistics: Human Language Technologies},
year      = {2021}
}
"""

_DATASETNAME = "pho_ner_covid"

_DESCRIPTION = """\
A named entity recognition dataset for Vietnamese with 10 newly-defined entity types in the context of the COVID-19 pandemic.
Data is extracted from news articles and manually annotated. In total, there are 34 984 entities over 10 027 sentences.
"""

_HOMEPAGE = "https://github.com/VinAIResearch/PhoNER_COVID19/tree/main"

_LANGUAGES = ["vie"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "word_level": {
            "dev": "https://raw.githubusercontent.com/VinAIResearch/PhoNER_COVID19/main/data/word/dev_word.json",
            "train": "https://raw.githubusercontent.com/VinAIResearch/PhoNER_COVID19/main/data/word/train_word.json",
            "test": "https://raw.githubusercontent.com/VinAIResearch/PhoNER_COVID19/main/data/word/test_word.json",
        },
        "syllable_level": {
            "dev": "https://raw.githubusercontent.com/VinAIResearch/PhoNER_COVID19/main/data/syllable/dev_syllable.json",
            "train": "https://raw.githubusercontent.com/VinAIResearch/PhoNER_COVID19/main/data/syllable/train_syllable.json",
            "test": "https://raw.githubusercontent.com/VinAIResearch/PhoNER_COVID19/main/data/syllable/test_syllable.json",
        },
    }
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SUPPORTED_SCHEMA_STRING_MAP: Dict[Tasks, str] = {}

for task, schema_string in zip(_SUPPORTED_TASKS, _SUPPORTED_SCHEMA_STRINGS):
    _SUPPORTED_SCHEMA_STRING_MAP[task] = schema_string

_SUBSETS = ["word_level", "syllable_level"]
_SPLITS = ["train", "dev", "test"]
_TAGS = [
    "O",
    "B-ORGANIZATION",
    "I-ORGANIZATION",
    "B-SYMPTOM_AND_DISEASE",
    "I-SYMPTOM_AND_DISEASE",
    "B-LOCATION",
    "B-DATE",
    "B-PATIENT_ID",
    "B-AGE",
    "B-NAME",
    "I-DATE",
    "B-JOB",
    "I-LOCATION",
    "B-TRANSPORTATION",
    "B-GENDER",
    "I-TRANSPORTATION",
    "I-JOB",
    "I-NAME",
    "I-AGE",
    "I-PATIENT_ID",
    "I-GENDER",
]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class PhoNerCovidDataset(datasets.GeneratorBasedBuilder):
    """A named entity recognition dataset for Vietnamese with 10 newly-defined entity types in the context of the COVID-19 pandemic."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = []

    for subset_id in _SUBSETS:
        BUILDER_CONFIGS.append(
            SEACrowdConfig(
                name=f"{subset_id}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} source schema",
                schema="source",
                subset_id=subset_id,
            )
        )

        seacrowd_schema_config: list[SEACrowdConfig] = []

        for seacrowd_schema in _SUPPORTED_SCHEMA_STRINGS:

            seacrowd_schema_config.append(
                SEACrowdConfig(
                    name=f"{subset_id}_{seacrowd_schema}",
                    version=SEACROWD_VERSION,
                    description=f"{_DATASETNAME} {seacrowd_schema} schema",
                    schema=f"{seacrowd_schema}",
                    subset_id=subset_id,
                )
            )

        BUILDER_CONFIGS.extend(seacrowd_schema_config)

    DEFAULT_CONFIG_NAME = f"{_SUBSETS[0]}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "words": datasets.Sequence(datasets.Value("string")),
                    "tags": datasets.Sequence(datasets.ClassLabel(names=_TAGS)),
                }
            )

        elif self.config.schema == _SUPPORTED_SCHEMA_STRING_MAP[Tasks.NAMED_ENTITY_RECOGNITION]:
            features = schemas.seq_label_features(label_names=_TAGS)

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        split_generators = []

        for split in _SPLITS:
            path = dl_manager.download_and_extract(_URLS[_DATASETNAME][self.config.subset_id][split])

            split_generators.append(
                datasets.SplitGenerator(
                    name=split,
                    gen_kwargs={
                        "path": path,
                    },
                )
            )

        return split_generators

    def _generate_examples(self, path: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        idx = 0
        df = pd.read_json(path, lines=True)

        if self.config.schema == "source":
            for _, row in df.iterrows():
                yield idx, row.to_dict()
                idx += 1

        elif self.config.schema == _SUPPORTED_SCHEMA_STRING_MAP[Tasks.NAMED_ENTITY_RECOGNITION]:
            df["id"] = df.index
            df = df.rename(columns={"words": "tokens", "tags": "labels"})

            for _, row in df.iterrows():
                yield idx, row.to_dict()
                idx += 1

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import Any, Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@misc{batsuren2022unimorph,
      title={UniMorph 4.0: Universal Morphology},
      author={
        Khuyagbaatar Batsuren and Omer Goldman and Salam Khalifa and Nizar
        Habash and Witold Kieraś and Gábor Bella and Brian Leonard and Garrett
        Nicolai and Kyle Gorman and Yustinus Ghanggo Ate and Maria Ryskina and
        Sabrina J. Mielke and Elena Budianskaya and Charbel El-Khaissi and Tiago
        Pimentel and Michael Gasser and William Lane and Mohit Raj and Matt
        Coler and Jaime Rafael Montoya Samame and Delio Siticonatzi Camaiteri
        and Benoît Sagot and Esaú Zumaeta Rojas and Didier López Francis and
        Arturo Oncevay and Juan López Bautista and Gema Celeste Silva Villegas
        and Lucas Torroba Hennigen and Adam Ek and David Guriel and Peter Dirix
        and Jean-Philippe Bernardy and Andrey Scherbakov and Aziyana Bayyr-ool
        and Antonios Anastasopoulos and Roberto Zariquiey and Karina Sheifer and
        Sofya Ganieva and Hilaria Cruz and Ritván Karahóǧa and Stella
        Markantonatou and George Pavlidis and Matvey Plugaryov and Elena
        Klyachko and Ali Salehi and Candy Angulo and Jatayu Baxi and Andrew
        Krizhanovsky and Natalia Krizhanovskaya and Elizabeth Salesky and Clara
        Vania and Sardana Ivanova and Jennifer White and Rowan Hall Maudslay and
        Josef Valvoda and Ran Zmigrod and Paula Czarnowska and Irene Nikkarinen
        and Aelita Salchak and Brijesh Bhatt and Christopher Straughn and Zoey
        Liu and Jonathan North Washington and Yuval Pinter and Duygu Ataman and
        Marcin Wolinski and Totok Suhardijanto and Anna Yablonskaya and Niklas
        Stoehr and Hossep Dolatian and Zahroh Nuriah and Shyam Ratan and Francis
        M. Tyers and Edoardo M. Ponti and Grant Aiton and Aryaman Arora and
        Richard J. Hatcher and Ritesh Kumar and Jeremiah Young and Daria
        Rodionova and Anastasia Yemelina and Taras Andrushko and Igor Marchenko
        and Polina Mashkovtseva and Alexandra Serova and Emily Prud'hommeaux and
        Maria Nepomniashchaya and Fausto Giunchiglia and Eleanor Chodroff and
        Mans Hulden and Miikka Silfverberg and Arya D. McCarthy and David
        Yarowsky and Ryan Cotterell and Reut Tsarfaty and Ekaterina Vylomova},
      year={2022},
      eprint={2205.03608},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "kod", "ceb", "hil", "tgl"]
_DATASETNAME = "unimorph"
_DESCRIPTION = """\
The Universal Morphology (UniMorph) project is a collaborative effort providing
broad-coverage instantiated normalized morphological inflection tables for
undreds of diverse world languages. The project comprises two major thrusts: a
language-independent feature schema for rich morphological annotation, and a
type-level resource of annotated data in diverse languages realizing that
schema. 5 Austronesian languages spoken in Southeast Asia, consisting 2
Malayo-Polynesian languages and 3 Greater Central Philippine languages, become
the part of UniMorph 4.0 release.
"""

_HOMEPAGE = "https://unimorph.github.io"
_LICENSE = Licenses.CC_BY_SA_3_0.value
_URL = "https://raw.githubusercontent.com/unimorph/"

_SUPPORTED_TASKS = [Tasks.MORPHOLOGICAL_INFLECTION]
_SOURCE_VERSION = "4.0.0"
_SEACROWD_VERSION = "1.0.0"


class UnimorphDataset(datasets.GeneratorBasedBuilder):
    """Unimorh 4.0 dataset by Batsuren et al., (2022)"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "pairs_multi"

    dataset_names = sorted([f"{_DATASETNAME}_{lang}" for lang in _LANGUAGES])
    BUILDER_CONFIGS = []
    for name in dataset_names:
        source_config = SEACrowdConfig(
            name=f"{name}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(source_config)
        seacrowd_config = SEACrowdConfig(
            name=f"{name}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(seacrowd_config)

    # Add configuration that allows loading all datasets at once.
    BUILDER_CONFIGS.extend(
        [
            # unimorph_source
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} source schema (all)",
                schema="source",
                subset_id=_DATASETNAME,
            ),
            # unimorph_seacrowd_pairs
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} SEACrowd schema (all)",
                schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
                subset_id=_DATASETNAME,
            ),
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"
    # https://huggingface.co/datasets/universal_morphologies/blob/main/universal_morphologies.py
    CLASS_CATEGORIES = {
        "Aktionsart": ["STAT", "DYN", "TEL", "ATEL", "PCT", "DUR", "ACH", "ACCMP", "SEMEL", "ACTY"],
        "Animacy": ["ANIM", "INAN", "HUM", "NHUM"],
        "Argument_Marking": [
            "ARGNO1S",
            "ARGNO2S",
            "ARGNO3S",
            "ARGNO1P",
            "ARGNO2P",
            "ARGNO3P",
            "ARGAC1S",
            "ARGAC2S",
            "ARGAC3S",
            "ARGAC1P",
            "ARGAC2P",
            "ARGAC3P",
            "ARGAB1S",
            "ARGAB2S",
            "ARGAB3S",
            "ARGAB1P",
            "ARGAB2P",
            "ARGAB3P",
            "ARGER1S",
            "ARGER2S",
            "ARGER3S",
            "ARGER1P",
            "ARGER2P",
            "ARGER3P",
            "ARGDA1S",
            "ARGDA2S",
            "ARGDA3S",
            "ARGDA1P",
            "ARGDA2P",
            "ARGDA3P",
            "ARGBE1S",
            "ARGBE2S",
            "ARGBE3S",
            "ARGBE1P",
            "ARGBE2P",
            "ARGBE3P",
        ],
        "Aspect": ["IPFV", "PFV", "PRF", "PROG", "PROSP", "ITER", "HAB"],
        "Case": [
            "NOM",
            "ACC",
            "ERG",
            "ABS",
            "NOMS",
            "DAT",
            "BEN",
            "PRP",
            "GEN",
            "REL",
            "PRT",
            "INS",
            "COM",
            "VOC",
            "COMPV",
            "EQTV",
            "PRIV",
            "PROPR",
            "AVR",
            "FRML",
            "TRANS",
            "BYWAY",
            "INTER",
            "AT",
            "POST",
            "IN",
            "CIRC",
            "ANTE",
            "APUD",
            "ON",
            "ONHR",
            "ONVR",
            "SUB",
            "REM",
            "PROXM",
            "ESS",
            "ALL",
            "ABL",
            "APPRX",
            "TERM",
        ],
        "Comparison": ["CMPR", "SPRL", "AB", "RL", "EQT"],
        "Definiteness": ["DEF", "INDF", "SPEC", "NSPEC"],
        "Deixis": ["PROX", "MED", "REMT", "REF1", "REF2", "NOREF", "PHOR", "VIS", "NVIS", "ABV", "EVEN", "BEL"],
        "Evidentiality": ["FH", "DRCT", "SEN", "VISU", "NVSEN", "AUD", "NFH", "QUOT", "RPRT", "HRSY", "INFER", "ASSUM"],
        "Finiteness": ["FIN", "NFIN"],
        "Gender": [
            "MASC",
            "FEM",
            "NEUT",
            "NAKH1",
            "NAKH2",
            "NAKH3",
            "NAKH4",
            "NAKH5",
            "NAKH6",
            "NAKH7",
            "NAKH8",
            "BANTU1",
            "BANTU2",
            "BANTU3",
            "BANTU4",
            "BANTU5",
            "BANTU6",
            "BANTU7",
            "BANTU8",
            "BANTU9",
            "BANTU10",
            "BANTU11",
            "BANTU12",
            "BANTU13",
            "BANTU14",
            "BANTU15",
            "BANTU16",
            "BANTU17",
            "BANTU18",
            "BANTU19",
            "BANTU20",
            "BANTU21",
            "BANTU22",
            "BANTU23",
        ],
        "Information_Structure": ["TOP", "FOC"],
        "Interrogativity": ["DECL", "INT"],
        "Language_Specific": [
            "LGSPEC1",
            "LGSPEC2",
            "LGSPEC3",
            "LGSPEC4",
            "LGSPEC5",
            "LGSPEC6",
            "LGSPEC7",
            "LGSPEC8",
            "LGSPEC9",
            "LGSPEC10",
        ],
        "Mood": [
            "IND",
            "SBJV",
            "REAL",
            "IRR",
            "AUPRP",
            "AUNPRP",
            "IMP",
            "COND",
            "PURP",
            "INTEN",
            "POT",
            "LKLY",
            "ADM",
            "OBLIG",
            "DEB",
            "PERM",
            "DED",
            "SIM",
            "OPT",
        ],
        "Number": ["SG", "PL", "GRPL", "DU", "TRI", "PAUC", "GRPAUC", "INVN"],
        "Part_Of_Speech": [
            "N",
            "PROPN",
            "ADJ",
            "PRO",
            "CLF",
            "ART",
            "DET",
            "V",
            "ADV",
            "AUX",
            "V.PTCP",
            "V.MSDR",
            "V.CVB",
            "ADP",
            "COMP",
            "CONJ",
            "NUM",
            "PART",
            "INTJ",
        ],
        "Person": ["0", "1", "2", "3", "4", "INCL", "EXCL", "PRX", "OBV"],
        "Polarity": ["POS", "NEG"],
        "Politeness": [
            "INFM",
            "FORM",
            "ELEV",
            "HUMB",
            "POL",
            "AVOID",
            "LOW",
            "HIGH",
            "STELEV",
            "STSUPR",
            "LIT",
            "FOREG",
            "COL",
        ],
        "Possession": [
            "ALN",
            "NALN",
            "PSS1S",
            "PSS2S",
            "PSS2SF",
            "PSS2SM",
            "PSS2SINFM",
            "PSS2SFORM",
            "PSS3S",
            "PSS3SF",
            "PSS3SM",
            "PSS1D",
            "PSS1DI",
            "PSS1DE",
            "PSS2D",
            "PSS2DM",
            "PSS2DF",
            "PSS3D",
            "PSS3DF",
            "PSS3DM",
            "PSS1P",
            "PSS1PI",
            "PSS1PE",
            "PSS2P",
            "PSS2PF",
            "PSS2PM",
            "PSS3PF",
            "PSS3PM",
        ],
        "Switch_Reference": ["SS", "SSADV", "DS", "DSADV", "OR", "SIMMA", "SEQMA", "LOG"],
        "Tense": ["PRS", "PST", "FUT", "IMMED", "HOD", "1DAY", "RCT", "RMT"],
        "Valency": ["IMPRS", "INTR", "TR", "DITR", "REFL", "RECP", "CAUS", "APPL"],
        "Voice": ["ACT", "MID", "PASS", "ANTIP", "DIR", "INV", "AGFOC", "PFOC", "LFOC", "BFOC", "ACFOC", "IFOC", "CFOC"],
    }

    TAG_TO_CAT = dict([(tag, cat) for cat, tags in CLASS_CATEGORIES.items() for tag in tags])
    CLASS_LABELS = [feat for _, category in CLASS_CATEGORIES.items() for feat in category]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "lemma": datasets.Value("string"),
                    "forms": datasets.Sequence(
                        dict(
                            [("word", datasets.Value("string"))]
                            + [(cat, datasets.Sequence(datasets.ClassLabel(names=tasks))) for cat, tasks in self.CLASS_CATEGORIES.items()]
                            + [("Other", datasets.Sequence(datasets.Value("string")))]  # for misspecified tags
                        )
                    ),
                }
            )

        if self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            all_features = [feat for _, category in self.CLASS_CATEGORIES.items() for feat in category]
            features = schemas.pairs_multi_features(label_names=self.CLASS_LABELS)

        return datasets.DatasetInfo(description=_DESCRIPTION, features=features, homepage=_HOMEPAGE, license=_LICENSE, citation=_CITATION)

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Return SplitGenerators."""
        source_data = []

        lang = self.config.name.split("_")[1]
        if lang in _LANGUAGES:
            # Load data per language
            source_data.append(dl_manager.download_and_extract(_URL + f"{lang}/main/{lang}"))
        else:
            # Load examples from all languages at once.
            for lang in _LANGUAGES:
                source_data.append(dl_manager.download_and_extract(_URL + f"{lang}/main/{lang}"))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": source_data,
                },
            )
        ]

    def _generate_examples(self, filepaths: List[Path]) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""

        all_forms: Dict[str, List[Dict[str, Any]]] = {}
        for source_file in filepaths:
            with open(source_file, encoding="utf-8") as file:
                for row in file:
                    if row.strip() == "" or row.strip().startswith("#"):
                        continue
                    lemma, word, tags = row.strip().split("\t")
                    all_forms[lemma] = all_forms.get(lemma, [])
                    tag_list = tags.replace("NDEF", "INDF").split(";")
                    form = dict([("word", word), ("Other", [])] + [(cat, []) for cat, tasks in self.CLASS_CATEGORIES.items()])
                    for tag_pre in tag_list:
                        tag = tag_pre.split("+")
                        if tag[0] in self.TAG_TO_CAT:
                            form[self.TAG_TO_CAT[tag[0]]] = tag
                        else:
                            form["Other"] += tag
                    all_forms[lemma] += [form]

        if self.config.schema == "source":
            for id_, (lemma, forms) in enumerate(all_forms.items()):
                res = {"lemma": lemma, "forms": {}}
                for k in ["word", "Other"] + list(self.CLASS_CATEGORIES.keys()):
                    res["forms"][k] = [form[k] for form in forms]
                yield id_, res

        if self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            idx = 0
            for lemma, forms in all_forms.items():
                for form in forms:
                    inflection = form.pop("word")
                    feats = [feat[0] for feat in list(form.values()) if feat and feat[0] in self.CLASS_LABELS]
                    example = {
                        "id": idx,
                        "text_1": lemma,
                        "text_2": inflection,
                        "label": feats,
                    }
                    idx += 1
                    yield idx, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This template serves as a starting point for contributing a dataset to the Nusantara Dataset repo.

When modifying it for your dataset, look for TODO items that offer specific instructions.

Full documentation on writing dataset loading scripts can be found here:
https://huggingface.co/docs/datasets/add_dataset.html

To create a dataset loading script you will create a class and implement 3 methods:
  * `_info`: Establishes the schema for the dataset, and returns a datasets.DatasetInfo object.
  * `_split_generators`: Downloads and extracts data for each split (e.g. train/val/test) or associate local data with each split.
  * `_generate_examples`: Creates examples from data on disk that conform to each schema defined in `_info`.

TODO: Before submitting your script, delete this doc string and replace it with a description of your dataset.

[seacrowd_schema_name] = (kb, pairs, qa, text, t2t, entailment)
"""
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

# TODO: Add BibTeX citation
_CITATION = """\
@article{nllb2022,
  author    = {NLLB Team, Marta R. Costa-jussà, James Cross, Onur Çelebi, Maha Elbayad, Kenneth Heafield, Kevin Heffernan, Elahe Kalbassi,  Janice Lam, Daniel Licht, Jean Maillard, Anna Sun, Skyler Wang, Guillaume Wenzek, Al Youngblood, Bapi Akula, Loic Barrault, Gabriel Mejia Gonzalez, Prangthip Hansanti, John Hoffman, Semarley Jarrett, Kaushik Ram Sadagopan, Dirk Rowe, Shannon Spruit, Chau Tran, Pierre Andrews, Necip Fazil Ayan, Shruti Bhosale, Sergey Edunov, Angela Fan, Cynthia Gao, Vedanuj Goswami, Francisco Guzmán, Philipp Koehn, Alexandre Mourachko, Christophe Ropers, Safiyyah Saleem, Holger Schwenk, Jeff Wang},
  title     = {No Language Left Behind: Scaling Human-Centered Machine Translation},
  year      = {2022}
}
"""

# TODO: create a module level variable with your dataset name (should match script name)
#  E.g. Hallmarks of Cancer: [dataset_name] --> hallmarks_of_cancer
_DATASETNAME = "nllb_seed"
_LANGUAGES = ["ace", "bjn", "bug", "eng"]
_LANGUAGE_MAP = {"ace": "Aceh", "bjn": "Banjar", "bug": "Bugis"}
_LANG_CODE_MAP = {"Aceh": "ace", "Banjar": "bjn", "Bugis": "bug"}
_LANGUAGE_PAIR = [("ace", "eng"), ("eng", "ace"), ("bjn", "eng"), ("eng", "bjn"), ("bug", "eng"), ("eng", "bug")]

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
No Language Left Behind Seed Data
NLLB Seed is a set of professionally-translated sentences in the Wikipedia domain. Data for NLLB-Seed was sampled from Wikimedia’s List of articles every Wikipedia should have, a collection of topics in different fields of knowledge and human activity. NLLB-Seed consists of around six thousand sentences in 39 languages. NLLB-Seed is meant to be used for training rather than model evaluation. Due to this difference, NLLB-Seed does not go through the human quality assurance process present in FLORES-200.
"""

# TODO: Add a link to an official homepage for the dataset here (if possible)
_HOMEPAGE = "https://github.com/facebookresearch/flores/tree/main/nllb_seed"

# TODO: Add the licence for the dataset here (if possible)
# Note that this doesn't have to be a common open source license.
# Some datasets have custom licenses. In this case, simply put the full license terms
# into `_LICENSE`
_LICENSE = "CC-BY-SA 4.0"

# TODO: Add links to the urls needed to download your dataset files.
#  For local datasets, this variable can be an empty dictionary.

# For publicly available datasets you will most likely end up passing these URLs to dl_manager in _split_generators.
# In most cases the URLs will be the same for the source and seacrowd config.
# However, if you need to access different files for each config you can have multiple entries in this dict.
# This can be an arbitrarily nested dict/list of URLs (see below in `_split_generators` method)
_URLS = {
    _DATASETNAME: "https://tinyurl.com/NLLBSeed",
}

# TODO: add supported task by dataset. One dataset may support multiple tasks
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

# TODO: set this to a version that is associated with the dataset. if none exists use "1.0.0"
#  This version doesn't have to be consistent with semantic versioning. Anything that is
#  provided by the original dataset as a version goes.
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"
_LOCAL = False


def seacrowd_config_constructor(lang, schema, version):
    if lang == "":
        raise ValueError(f"Invalid lang {lang}")

    if schema != "source" and schema != "seacrowd_t2t":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="nllb_seed_{lang}_{schema}".format(lang=lang, schema=schema),
        version=datasets.Version(version),
        description="nllb_seed {schema} schema for {lang} language".format(lang=_LANGUAGE_MAP[lang], schema=schema),
        schema=schema,
        subset_id="nllb_seed",
    )


# TODO: Name the dataset class to match the script name using CamelCase instead of snake_case
class NLLBSeed(datasets.GeneratorBasedBuilder):
    """TODO: Short description of my dataset."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    # You will be able to load the "source" or "seacrowd" configurations with
    # ds_source = datasets.load_dataset('my_dataset', name='source')
    # ds_seacrowd = datasets.load_dataset('my_dataset', name='seacrowd')

    # For local datasets you can make use of the `data_dir` and `data_files` kwargs
    # https://huggingface.co/docs/datasets/add_dataset.html#downloading-data-files-and-organizing-splits
    # ds_source = datasets.load_dataset('my_dataset', name='source', data_dir="/path/to/data/files")
    # ds_seacrowd = datasets.load_dataset('my_dataset', name='seacrowd', data_dir="/path/to/data/files")

    # TODO: For each dataset, implement Config for Source and Nusantara;
    #  If dataset contains more than one subset (see seacrowd/sea_datasets/smsa.py) implement for EACH of them.
    #  Each of them should contain:
    #   - name: should be unique for each dataset config eg. smsa_(source|seacrowd)_[seacrowd_schema_name]
    #   - version: option = (SOURCE_VERSION|SEACROWD_VERSION)
    #   - description: one line description for the dataset
    #   - schema: options = (source|seacrowd_[seacrowd_schema_name])
    #   - subset_id: subset id is the canonical name for the dataset (eg. smsa)
    #  where [seacrowd_schema_name] = (kb, pairs, qa, text, t2t)

    BUILDER_CONFIGS = [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in _LANGUAGE_MAP] + [seacrowd_config_constructor(lang, "seacrowd_t2t", _SEACROWD_VERSION) for lang in _LANGUAGE_MAP]
    """
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="nllb_seed_source",
            version=SOURCE_VERSION,
            description="nllb_seed source schema",
            schema="source",
            subset_id="nllb_seed",
        ),
        SEACrowdConfig(
            name="nllb_seed_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="nllb_seed Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="nllb_seed",
        ),
    ]
    """
    DEFAULT_CONFIG_NAME = "nllb_seed_ace_source"

    def _info(self) -> datasets.DatasetInfo:

        # Create the source schema; this schema will keep all keys/information/labels as close to the original dataset as possible.

        # You can arbitrarily nest lists and dictionaries.
        # For iterables, use lists over tuples or `datasets.Sequence`

        if self.config.schema == "source":
            # TODO: Create your source schema here
            # raise NotImplementedError()

            # EX: Arbitrary NER type dataset
            # features = datasets.Features(
            #    {
            #        "doc_id": datasets.Value("string"),
            #        "text": datasets.Value("string"),
            #        "entities": [
            #            {
            #                "offsets": [datasets.Value("int64")],
            #                "text": datasets.Value("string"),
            #                "type": datasets.Value("string"),
            #                "entity_id": datasets.Value("string"),
            #            }
            #        ],
            #    }
            # )
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src": [datasets.Value("string")],
                    "tgt": [datasets.Value("string")],
                }
            )

        # Choose the appropriate seacrowd schema for your task and copy it here. You can find information on the schemas in the CONTRIBUTING guide.

        # In rare cases you may get a dataset that supports multiple tasks requiring multiple schemas. In that case you can define multiple seacrowd configs with a seacrowd_[seacrowd_schema_name] format.

        # For example seacrowd_kb, seacrowd_t2t
        elif self.config.schema == "seacrowd_t2t":
            # e.g. features = schemas.kb_features
            features = schemas.text2text_features
            # TODO: Choose your seacrowd schema here
            # raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration

        # If you need to access the "source" or "seacrowd" config choice, that will be in self.config.name

        # LOCAL DATASETS: You do not need the dl_manager; you can ignore this argument. Make sure `gen_kwargs` in the return gets passed the right filepath

        # PUBLIC DATASETS: Assign your data-dir based on the dl_manager.

        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs; many examples use the download_and_extract method; see the DownloadManager docs here: https://huggingface.co/docs/datasets/package_reference/builder_classes.html#datasets.DownloadManager

        # dl_manager can accept any type of nested list/dict and will give back the same structure with the url replaced with the path to local files.

        # TODO: KEEP if your dataset is PUBLIC; remove if not
        urls = _URLS[_DATASETNAME]
        data_dir = Path(dl_manager.download_and_extract(urls)) / "NLLB-Seed"
        data_subdir = {"ace": os.path.join(data_dir, "ace_Latn-eng_Latn"), "bjn": os.path.join(data_dir, "bjn_Latn-eng_Latn"), "bug": os.path.join(data_dir, "bug_Latn-eng_Latn")}
        lang = self.config.name.split("_")[2]

        """
        # TODO: KEEP if your dataset is LOCAL; remove if NOT
        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

        # Not all datasets have predefined canonical train/val/test splits.
        # If your dataset has no predefined splits, use datasets.Split.TRAIN for all of the data.
        """

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": {lang: os.path.join(data_subdir[lang], lang + "_Latn"), lang + "_eng": os.path.join(data_subdir[lang], "eng_Latn"), "split": "train"}},
            )
        ]

    # method parameters are unpacked from `gen_kwargs` as given in `_split_generators`

    # TODO: change the args of this function to match the keys in `gen_kwargs`. You may add any necessary kwargs.

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # TODO: This method handles input defined in _split_generators to yield (key, example) tuples from the dataset.

        # The `key` is for legacy reasons (tfds) and is not important in itself, but must be unique for each example.

        # NOTE: For local datasets you will have access to self.config.data_dir and self.config.data_files
        lang = self.config.name.split("_")[2]
        lang_code = lang

        eng_text = open(filepath[lang_code + "_eng"], "r").readlines()
        lang_text = open(filepath[lang_code], "r").readlines()

        eng_text = list(map(str.strip, eng_text))
        lang_text = list(map(str.strip, lang_text))
        """
        if self.config.schema == "source":
            # TODO: yield (key, example) tuples in the original dataset schema
            for key, example in thing:
                yield key, example

        elif self.config.schema == "seacrowd_t2t":
            # TODO: yield (key, example) tuples in the seacrowd schema
            for key, example in thing:
                yield key, example
        """
        if self.config.schema == "source":
            for id, (src, tgt) in enumerate(zip(lang_text, eng_text)):
                row = {
                    "id": str(id),
                    "src": [src],
                    "tgt": [tgt],
                }
                yield id, row

        elif self.config.schema == "seacrowd_t2t":
            for id, (src, tgt) in enumerate(zip(lang_text, eng_text)):
                row = {
                    "id": str(id),
                    "text_1": src,
                    "text_2": tgt,
                    "text_1_name": lang_code,
                    "text_2_name": "eng",
                }
                yield id, row
        else:
            raise ValueError(f"Invalid config: {self.config.name}")


# This template is based on the following template from the datasets package:
# https://github.com/huggingface/datasets/blob/master/templates/new_dataset_script.py


# This allows you to run your dataloader with `python [dataset_name].py` during development
# TODO: Remove this before making your PR
if __name__ == "__main__":
    datasets.load_dataset(__file__)



from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@INPROCEEDINGS{8459963,
      author={E. D. {Livelo} and C. {Cheng}},
      booktitle={2018 IEEE International Conference on Agents (ICA)},
      title={Intelligent Dengue Infoveillance Using Gated Recurrent Neural Learning and Cross-Label Frequencies},
      year={2018},
      volume={},
      number={},
      pages={2-7},
      doi={10.1109/AGENTS.2018.8459963}}
    }
"""

_LANGUAGES = ["fil"]

# copied from https://huggingface.co/datasets/dengue_filipino/blob/main/dengue_filipino.py
_URL = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/dengue/dengue_raw.zip"
_DATASETNAME = "dengue_filipino"

_DESCRIPTION = """\
Benchmark dataset for low-resource multi-label classification, with 4,015 training, 500 testing, and 500 validation examples, each labeled as part of five classes. Each sample can be a part of multiple classes. Collected as tweets.
"""

_HOMEPAGE = "https://github.com/jcblaisecruz02/Filipino-Text-Benchmarks"

_LICENSE = Licenses.UNKNOWN.value

_SUPPORTED_TASKS = [Tasks.DOMAIN_KNOWLEDGE_MULTICLASSIFICATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_LOCAL = False


class DengueFilipinoDataset(datasets.GeneratorBasedBuilder):
    """Dengue Dataset Low-Resource Multi-label Text Classification Dataset in Filipino"""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text_multi",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema text multi",
            schema="seacrowd_text_multi",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "absent": datasets.features.ClassLabel(names=["0", "1"]),
                    "dengue": datasets.features.ClassLabel(names=["0", "1"]),
                    "health": datasets.features.ClassLabel(names=["0", "1"]),
                    "mosquito": datasets.features.ClassLabel(names=["0", "1"]),
                    "sick": datasets.features.ClassLabel(names=["0", "1"]),
                }
            )
        elif self.config.schema == "seacrowd_text_multi":
            features = schemas.text_multi_features(["0", "1"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "split": "validation",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, split: str) -> Tuple[int, Dict]:
        dataset = datasets.load_dataset(_DATASETNAME, split=split)
        for id, data in enumerate(dataset):
            if self.config.schema == "source":
                yield id, {
                    "text": data["text"],
                    "absent": data["absent"],
                    "dengue": data["dengue"],
                    "health": data["health"],
                    "mosquito": data["mosquito"],
                    "sick": data["sick"],
                }

            elif self.config.schema == "seacrowd_text_multi":
                yield id, {
                    "id": id,
                    "text": data["text"],
                    "labels": [
                        data["absent"],
                        data["dengue"],
                        data["health"],
                        data["mosquito"],
                        data["sick"],
                    ],
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from itertools import chain, repeat
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_ud_data, load_ud_data_as_seacrowd_kb
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@conference{2f8c7438a7f44f6b85b773586cff54e8,
    title = "A gold standard dependency treebank for Indonesian",
    author = "Ika Alfina and Arawinda Dinakaramani and Fanany, {Mohamad Ivan} and Heru Suhartanto",
    note = "Publisher Copyright: {\textcopyright} 2019 Proceedings of the 33rd Pacific Asia Conference on Language, Information and Computation, PACLIC 2019. All rights reserved.; \
33rd Pacific Asia Conference on Language, Information and Computation, PACLIC 2019 ; Conference date: 13-09-2019 Through 15-09-2019",
    year = "2019",
    month = jan,
    day = "1",
    language = "English",
    pages = "1--9",
}

@article{DBLP:journals/corr/abs-2011-00677,
    author    = {Fajri Koto and
                 Afshin Rahimi and
                 Jey Han Lau and
                 Timothy Baldwin},
    title     = {IndoLEM and IndoBERT: {A} Benchmark Dataset and Pre-trained Language
                 Model for Indonesian {NLP}},
    journal   = {CoRR},
    volume    = {abs/2011.00677},
    year      = {2020},
    url       = {https://arxiv.org/abs/2011.00677},
    eprinttype = {arXiv},
    eprint    = {2011.00677},
    timestamp = {Fri, 06 Nov 2020 15:32:47 +0100},
    biburl    = {https://dblp.org/rec/journals/corr/abs-2011-00677.bib},
    bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "indolem_ud_id_pud"

_DESCRIPTION = """\
1 of 8 sub-datasets of IndoLEM, a comprehensive dataset encompassing 7 NLP tasks (Koto et al., 2020).
This dataset is part of [Parallel Universal Dependencies (PUD)](http://universaldependencies.org/conll17/) project.
This is based on the first corrected version by Alfina et al. (2019), contains 1,000 sentences.
"""

_HOMEPAGE = "https://indolem.github.io/"

_LICENSE = "Creative Commons Attribution 4.0"

_FOLDS = list(range(5))
_DEFAULT_FOLD = _FOLDS[0]
_URLS = {
    f"{_DATASETNAME}_{fold}": {
        "train": f"https://raw.githubusercontent.com/indolem/indolem/main/dependency_parsing/UD_Indonesian_PUD/folds/train{fold}.conllu",
        "validation": f"https://raw.githubusercontent.com/indolem/indolem/main/dependency_parsing/UD_Indonesian_PUD/folds/dev{fold}.conllu",
        "test": f"https://raw.githubusercontent.com/indolem/indolem/main/dependency_parsing/UD_Indonesian_PUD/folds/test{fold}.conllu",
    }
    for fold in _FOLDS
}

_SUPPORTED_TASKS = [Tasks.DEPENDENCY_PARSING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IndolemUDIDPUDDataset(datasets.GeneratorBasedBuilder):
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = list(
        chain(
            (
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_source",
                    version=SOURCE_VERSION,
                    description=f"{_DATASETNAME} default fold ('{_DEFAULT_FOLD}') of source schema",
                    schema="source",
                    subset_id=f"{_DATASETNAME}_{_DEFAULT_FOLD}",
                ),
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_seacrowd_kb",
                    version=SEACROWD_VERSION,
                    description=f"{_DATASETNAME} default fold ('{_DEFAULT_FOLD}') of Nusantara KB schema",
                    schema="seacrowd_kb",
                    subset_id=f"{_DATASETNAME}_{_DEFAULT_FOLD}",
                ),
            ),
            *(
                (
                    SEACrowdConfig(
                        name=f"{_DATASETNAME}_{fold}_source",
                        version=ver_src,
                        description=f"{_DATASETNAME} fold '{fold}' of source schema",
                        schema="source",
                        subset_id=f"{_DATASETNAME}_{fold}",
                    ),
                    SEACrowdConfig(
                        name=f"{_DATASETNAME}_{fold}_seacrowd_kb",
                        version=ver_nusa,
                        description=f"{_DATASETNAME} fold '{fold}' of Nusantara KB schema",
                        schema="seacrowd_kb",
                        subset_id=f"{_DATASETNAME}_{fold}",
                    ),
                )
                for fold, ver_src, ver_nusa in zip(_FOLDS, repeat(SOURCE_VERSION), repeat(SEACROWD_VERSION))
            ),
        )
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    # metadata
                    "sent_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "text_en": datasets.Value("string"),
                    # tokens
                    "id": [datasets.Value("string")],
                    "form": [datasets.Value("string")],
                    "lemma": [datasets.Value("string")],
                    "upos": [datasets.Value("string")],  # Alternatively, use ClassLabel (https://huggingface.co/datasets/universal_dependencies/blob/main/universal_dependencies.py#L1211)
                    "xpos": [datasets.Value("string")],
                    "feats": [datasets.Value("string")],
                    "head": [datasets.Value("string")],
                    "deprel": [datasets.Value("string")],
                    "deps": [datasets.Value("string")],
                    "misc": [datasets.Value("string")],
                }
            )

        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[self.config.subset_id]
        data_dir = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # method parameters are unpacked from `gen_kwargs` as given in `_split_generators`

        try:
            generator_fn = {
                "source": load_ud_data,
                "seacrowd_kb": load_ud_data_as_seacrowd_kb,
            }[self.config.schema]
        except KeyError:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        for key, example in enumerate(generator_fn(filepath)):
            yield key, example


# if __name__ == "__main__":
#     datasets.load_dataset(__file__, name=f"indolem_ud_id_pud_source")
#     datasets.load_dataset(__file__, name=f"indolem_ud_id_pud_seacrowd_kb")
#     for fold in _FOLDS:
#         datasets.load_dataset(__file__, name=f"indolem_ud_id_pud_{fold}_source")
#         datasets.load_dataset(__file__, name=f"indolem_ud_id_pud_{fold}_seacrowd_kb")



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import json
import os

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from zipfile import ZipFile

_CITATION = """\
@inproceedings{sani-cocosda-2012,
    title = "Towards Language Preservation: Preliminary Collection and Vowel Analysis of {I}ndonesian Ethnic Speech Data",
    author = "Sani, Auliya and Sakti, Sakriani and Neubig, Graham and Toda, Tomoki and Mulyanto, Adi and Nakamura, Satoshi",
    booktitle = "Proc. Oriental COCOSDA",
    year = "2012",
    pages = "118--122"
    address = "Macau, China"
}
"""

_LOCAL = False
_LANGUAGES = ["sun", "jav"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indspeech_news_ethnicsr"

_DESCRIPTION = """
INDspeech_NEWS_EthnicSR is a collection of Indonesian ethnic speech corpora for Javanese and Sundanese for Indonesian ethnic speech recognition. It was developed in 2012 by the Nara Institute of Science and Technology (NAIST, Japan) in collaboration with the Bandung Institute of Technology (ITB, Indonesia) [Sani et al., 2012].
"""

_HOMEPAGE = "https://github.com/s-sakti/data_indsp_news_ethnicsr"

_LICENSE = "CC-BY-NC-SA 4.0"

_URLS = {
    _DATASETNAME: "https://github.com/s-sakti/data_indsp_news_ethnicsr/archive/refs/heads/main.zip",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndSpeechNewsEthnicSR(datasets.GeneratorBasedBuilder):
    """INDspeech_NEWS_EthnicSR is a collection of Indonesian ethnic speech corpora for Javanese and Sundanese for Indonesian ethnic speech recognition. It was developed in 2012 by the Nara Institute of Science and Technology (NAIST, Japan) in collaboration with the Bandung Institute of Technology (ITB, Indonesia) [Sani et al., 2012]."""


    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    
    BUILDER_CONFIGS = []
    
    for fold_id in ["overlap", "nooverlap"]:
        for fold_name in ['jv', "su"]:
            BUILDER_CONFIGS.extend(
                [SEACrowdConfig(
                    name=f"indspeech_news_ethnicsr_{fold_name}_{fold_id}_source",
                    version=_SOURCE_VERSION,
                    description="indspeech_news_ethnicsr source schema",
                    schema="source",
                    subset_id=f"indspeech_news_ethnicsr_{fold_name}_{fold_id}"
                ),
                SEACrowdConfig(
                    name=f"indspeech_news_ethnicsr_{fold_name}_{fold_id}_seacrowd_sptext",
                    version=_SOURCE_VERSION,
                    description="indspeech_news_ethnicsr Nusantara schema",
                    schema="seacrowd_sptext",
                    subset_id=f"indspeech_news_ethnicsr_{fold_name}_{fold_id}"
                ),]
            ) 
            
    DEFAULT_CONFIG_NAME = "indspeech_news_ethnicsr_jv_nooverlap_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
               {
                   "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
               }
            )

        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="text")],
        )


    def _get_fold_name_id(self):
        subset_id = self.config.subset_id
        subset_id_list = subset_id.split('_')
        fold_name = subset_id_list[-2]
        fold_id = subset_id_list[-1]
        if fold_id == "overlap":
            fold_id = 1
        elif fold_id == "nooverlap":
            fold_id = 2
        return fold_name, fold_id
    
    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        fold_name, fold_id = self._get_fold_name_id()
        if fold_name == 'su':
            fold_name1 = "Sunda"
            fold_name2 = 'Snd'
            
        else:
            fold_name1 = 'Jawa'
            fold_name2 = 'Jaw'
        
        urls = _URLS[_DATASETNAME]
        data_dir = Path(dl_manager.download_and_extract(urls))
#         print("data_dir", data_dir)
        text_file = os.path.join(data_dir, f"data_indsp_news_ethnicsr-main/{fold_name1}/text/transcript.txt")
        wav_folder = os.path.join(data_dir, f"data_indsp_news_ethnicsr-main/{fold_name1}/speech/16kHz/")
        train_list = os.path.join(data_dir, f"data_indsp_news_ethnicsr-main/{fold_name1}/lst/dataset{fold_id}_train_news_{fold_name2}.lst")
        test_list = os.path.join(data_dir, f"data_indsp_news_ethnicsr-main/{fold_name1}/lst/dataset{fold_id}_test_news_{fold_name2}.lst")
        
        #unzip        
        for speaker_id in range(1, 11):
            speaker_id = "%03d" % (speaker_id)
            zip_file = os.path.join(wav_folder, f"{fold_name2}{speaker_id}.zip")
            out_folder = os.path.join(wav_folder, f"{fold_name2}{speaker_id}")
            if not os.path.exists(out_folder):
                with ZipFile(zip_file, 'r') as f:
                    f.extractall(out_folder)
                    
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,

                gen_kwargs={
                    "wav_folder": wav_folder,
                    "text_path": text_file,
                    "split": "train",
                    "fold_name": fold_name,
                    "file_list": train_list,
                },
            ),
            
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "wav_folder": wav_folder,
                    "text_path": text_file,
                    "split": "test",
                    "fold_name": fold_name,
                    "file_list": test_list,
                },
            )
        ]
    

    def _generate_examples(self, wav_folder: Path, text_path: Path, split: str, fold_name: str, file_list: Path) -> Tuple[int, Dict]:
        if fold_name == 'su':
            fold_name2 = 'Snd'
        else:
            fold_name2 = 'Jaw'
            
        id2text = {}
        with open(text_path, "r", encoding='unicode_escape') as f:
            for text_idx, line in enumerate(f.readlines()):
                id2text.update({"%04d" % (text_idx + 1):line.strip()})
                
        
        wave_list = []
        with open(file_list) as f:
            for l in f.readlines():
                audio_id = l.strip()[:-4]
                speaker_id = audio_id.split('_')[0][-3:]
                text_id = audio_id.split('_')[-1]
                text = id2text[text_id]
                
                wav_path = os.path.join(wav_folder, audio_id.split('_')[0], l.strip())
                if not os.path.exists(wav_path):
                    print('no exisit wav_path', wav_path)
                assert os.path.exists(wav_path)
                
                if self.config.schema == "source":
                    ex = {
                        "id": audio_id,
                        "speaker_id": speaker_id,
                        "path": wav_path,
                        "audio": wav_path,
                        "text": text,
                    }
                    yield audio_id, ex

                elif self.config.schema == "seacrowd_sptext":
                    ex = {
                        "id": audio_id,
                        "speaker_id": speaker_id,
                        "path": wav_path,
                        "audio": wav_path,
                        "text": text,
                        "metadata": {
                            "speaker_age": None,
                            "speaker_gender": audio_id.split("_")[1],
                        }
                    }
                    yield audio_id, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" NERGrit Dataset """

from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@misc{Fahmi_NERGRIT_CORPUS_2019,
author = {Fahmi, Husni and Wibisono, Yudi and Kusumawati, Riyanti},
title = {{NERGRIT CORPUS}},
url = {https://github.com/grit-id/nergrit-corpus},
year = {2019}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "nergrit"
_DESCRIPTION = """\
Nergrit Corpus is a dataset collection of Indonesian Named Entity Recognition (NER), Statement Extraction,
and Sentiment Analysis developed by PT Gria Inovasi Teknologi (GRIT).
The Named Entity Recognition contains 18 entities as follow:
    'CRD': Cardinal
    'DAT': Date
    'EVT': Event
    'FAC': Facility
    'GPE': Geopolitical Entity
    'LAW': Law Entity (such as Undang-Undang)
    'LOC': Location
    'MON': Money
    'NOR': Political Organization
    'ORD': Ordinal
    'ORG': Organization
    'PER': Person
    'PRC': Percent
    'PRD': Product
    'QTY': Quantity
    'REG': Religion
    'TIM': Time
    'WOA': Work of Art
    'LAN': Language
"""

_HOMEPAGE = "https://github.com/grit-id/nergrit-corpus"
_LICENSE = "MIT"
_URL = "https://github.com/cahya-wirawan/indonesian-language-models/raw/master/data/nergrit-corpus_20190726_corrected.tgz"
_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class NergritDataset(datasets.GeneratorBasedBuilder):
    """Indonesian Named Entity Recognition from https://github.com/grit-id/nergrit-corpus."""

    label_classes = {
        "ner": [
            "B-CRD",
            "B-DAT",
            "B-EVT",
            "B-FAC",
            "B-GPE",
            "B-LAN",
            "B-LAW",
            "B-LOC",
            "B-MON",
            "B-NOR",
            "B-ORD",
            "B-ORG",
            "B-PER",
            "B-PRC",
            "B-PRD",
            "B-QTY",
            "B-REG",
            "B-TIM",
            "B-WOA",
            "I-CRD",
            "I-DAT",
            "I-EVT",
            "I-FAC",
            "I-GPE",
            "I-LAN",
            "I-LAW",
            "I-LOC",
            "I-MON",
            "I-NOR",
            "I-ORD",
            "I-ORG",
            "I-PER",
            "I-PRC",
            "I-PRD",
            "I-QTY",
            "I-REG",
            "I-TIM",
            "I-WOA",
            "O",
        ],
        "sentiment": ["B-POS", "B-NEG", "B-NET", "I-POS", "I-NEG", "I-NET", "O"],
        "statement": ["B-BREL", "B-FREL", "B-STAT", "B-WHO", "I-BREL", "I-FREL", "I-STAT", "I-WHO", "O"],
    }
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"nergrit_{task}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="NERGrit source schema",
            schema="source",
            subset_id=f"nergrit_{task}",
        )
        for task in label_classes
    ]
    BUILDER_CONFIGS += [
        SEACrowdConfig(
            name=f"nergrit_{task}_seacrowd_seq_label",
            version=datasets.Version(_SOURCE_VERSION),
            description="NERGrit Nusantara schema",
            schema="seacrowd_seq_label",
            subset_id=f"nergrit_{task}",
        )
        for task in label_classes
    ]

    DEFAULT_CONFIG_NAME = "nergrit_ner_source"

    def _info(self):
        features = None
        task = self.config.subset_id.split("_")[-1]
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "tokens": [datasets.Value("string")], "ner_tag": [datasets.Value("string")]})
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes[task])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        task = self.config.subset_id.split("_")[-1]
        archive = Path(dl_manager.download_and_extract(_URL))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": archive / f"nergrit-corpus/{task}/data/train_corrected.txt"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": archive / f"nergrit-corpus/{task}/data/test_corrected.txt"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": archive / f"nergrit-corpus/{task}/data/valid_corrected.txt"},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        conll_dataset = load_conll_data(filepath)

        if self.config.schema == "source":
            for index, row in enumerate(conll_dataset):
                ex = {"index": str(index), "tokens": row["sentence"], "ner_tag": row["label"]}
                yield index, ex
        elif self.config.schema == "seacrowd_seq_label":
            for index, row in enumerate(conll_dataset):
                ex = {"id": str(index), "tokens": row["sentence"], "labels": row["label"]}
                yield index, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from dataclasses import dataclass

import datasets


@dataclass
class SEACrowdConfig(datasets.BuilderConfig):
    """BuilderConfig for SEACrowd."""

    name: str = None
    version: datasets.Version = None
    description: str = None
    schema: str = None
    subset_id: str = None



from pathlib import Path
from typing import List

import datasets
import pandas as pd
import codecs
from collections import namedtuple

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Tasks

_DATASETNAME = "barasa"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 langauge code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{baccianella-etal-2010-sentiwordnet,
    title = "{S}enti{W}ord{N}et 3.0: An Enhanced Lexical Resource for Sentiment Analysis and Opinion Mining",
    author = "Baccianella, Stefano  and
      Esuli, Andrea  and
      Sebastiani, Fabrizio",
    booktitle = "Proceedings of the Seventh International Conference on Language Resources and Evaluation ({LREC}'10)",
    month = may,
    year = "2010",
    address = "Valletta, Malta",
    publisher = "European Language Resources Association (ELRA)",
    url = "http://www.lrec-conf.org/proceedings/lrec2010/pdf/769_Paper.pdf",
    abstract = "In this work we present SENTIWORDNET 3.0, a lexical resource explicitly devised for supporting sentiment classification and opinion mining applications. SENTIWORDNET 3.0 is an improved version of SENTIWORDNET 1.0, a lexical resource publicly available for research purposes, now currently licensed to more than 300 research groups and used in a variety of research projects worldwide. Both SENTIWORDNET 1.0 and 3.0 are the result of automatically annotating all WORDNET synsets according to their degrees of positivity, negativity, and neutrality. SENTIWORDNET 1.0 and 3.0 differ (a) in the versions of WORDNET which they annotate (WORDNET 2.0 and 3.0, respectively), (b) in the algorithm used for automatically annotating WORDNET, which now includes (additionally to the previous semi-supervised learning step) a random-walk step for refining the scores. We here discuss SENTIWORDNET 3.0, especially focussing on the improvements concerning aspect (b) that it embodies with respect to version 1.0. We also report the results of evaluating SENTIWORDNET 3.0 against a fragment of WORDNET 3.0 manually annotated for positivity, negativity, and neutrality; these results indicate accuracy improvements of about 20{\%} with respect to SENTIWORDNET 1.0.",
}

@misc{moeljadi_2016,
    title={Neocl/Barasa: Indonesian SentiWordNet},
    url={https://github.com/neocl/barasa},
    journal={GitHub},
    author={Moeljadi, David},
    year={2016}, month={Mar}
}
"""

_DESCRIPTION = """\
The Barasa dataset is an Indonesian SentiWordNet for sentiment analysis.
For each term, the pair (POS,ID) uniquely identifies a WordNet (3.0) synset and there are PosScore and NegScore to show the positivity and negativity of the term.
The objectivity score can be calculated as: ObjScore = 1 - (PosScore + NegScore).
"""

_HOMEPAGE = "https://github.com/neocl/barasa"

_LICENSE = "MIT"

_URLs = {
    "senti_wordnet": "https://github.com/neocl/barasa/raw/master/data/SentiWordNet_3.0.0_20130122.txt",
    "tab": "https://github.com/neocl/barasa/raw/55f669ca3e417e7fa8d0ebafb67700b9c9eeff1d/data/wn-msa-all.tab",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = None

class Barasa(datasets.GeneratorBasedBuilder):

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="barasa_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Barasa source schema",
            schema="source",
            subset_id="barasa",
        ),
    ]

    DEFAULT_CONFIG_NAME = "barasa_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "synset": datasets.Value("string"),
                    "PosScore": datasets.Value("float32"),
                    "NegScore": datasets.Value("float32"),
                    "language": datasets.Value("string"),
                    "goodness": datasets.Value("string"),
                    "lemma": datasets.Value("string"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        sentiWordnet_tsv_path = Path(dl_manager.download_and_extract(_URLs["senti_wordnet"]))
        tab_path = Path(dl_manager.download_and_extract(_URLs["tab"]))
        data_files = {
            "sentiWordnet": sentiWordnet_tsv_path,
            "tab": tab_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": [data_files["sentiWordnet"], data_files["tab"]]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        lines = self.gen_barasa(filepath[0], filepath[1])

        if self.config.schema == "source":
            for i, row in enumerate(lines):
                synset, language, goodness, lemma, PosScore, NegScore = row.split('\t')[:6]
                PosScore = float(PosScore)
                NegScore = float(NegScore)
                ex = {
                    "index": i,
                    "synset": synset,
                    "PosScore": PosScore,
                    "NegScore": NegScore,
                    "language": language,
                    "goodness": goodness,
                    "lemma": lemma,
                }
                yield i, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")

    def gen_barasa(self, SENTI_WORDNET_FILE, BAHASA_WORDNET_FILE):
        SynsetInfo = namedtuple('SynsetInfo', ['synset', 'pos', 'neg'])
        LemmaInfo  = namedtuple('LemmaInfo', ['lemma', 'pos', 'neg'])

        SYNSET_SCORE = {}
        LEMMA_SCORE = {}

        with codecs.open(SENTI_WORDNET_FILE, encoding='utf-8', mode='r') as SentiWN:
            for line in SentiWN.readlines():
                if line.startswith('#') or len(line.strip()) == 0: # ignore comments
                    continue
                # strip off end-of-line, then split
                pos, snum, pscore, nscore, lemma, definition = line.strip().split('\t')
                synset = '%s-%s' % (snum, pos)
                SYNSET_SCORE[synset] = SynsetInfo(synset, pscore, nscore)

        newlines = []
        with codecs.open(BAHASA_WORDNET_FILE, encoding='utf-8', mode='r') as BahasaWN:
            for line in BahasaWN.readlines():
                synset, lang, goodness, lemma = line.strip().split('\t')
                if synset in SYNSET_SCORE:
                    sscore = SYNSET_SCORE[synset]
                    LEMMA_SCORE[lemma] = LemmaInfo(lemma, sscore.pos, sscore.neg)
                    newline = ("%s\t" * 6) % (synset, lang, goodness, lemma, sscore.pos, sscore.neg)
                newlines.append(newline)

        return newlines


"""
General ImageText Classification Schema
"""
import datasets

def features(label_names = ["Yes", "No"]):
    return datasets.Features(
        {
            "id": datasets.Value("string"),
            "image_paths": datasets.Sequence(datasets.Value("string")),
            "texts": datasets.Value("string"),
            "metadata": {
                "context": datasets.Value("string"),
                "labels": datasets.Sequence(datasets.ClassLabel(names=label_names)),
            }
        }
    )




import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = r"""
@inproceedings{cruz-etal-2020-localization,
    title = "Localization of Fake News Detection via Multitask Transfer Learning",
    author = "Cruz, Jan Christian Blaise  and
      Tan, Julianne Agatha  and
      Cheng, Charibeth",
    editor = "Calzolari, Nicoletta  and
      B{\'e}chet, Fr{\'e}d{\'e}ric  and
      Blache, Philippe  and
      Choukri, Khalid  and
      Cieri, Christopher  and
      Declerck, Thierry  and
      Goggi, Sara  and
      Isahara, Hitoshi  and
      Maegaard, Bente  and
      Mariani, Joseph  and
      Mazo, H{\'e}l{\`e}ne  and
      Moreno, Asuncion  and
      Odijk, Jan  and
      Piperidis, Stelios",
    booktitle = "Proceedings of the Twelfth Language Resources and Evaluation Conference",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2020.lrec-1.316",
    pages = "2596--2604",
    language = "English",
    ISBN = "979-10-95546-34-4",
}
"""

_LOCAL = False
_LANGUAGES = ["fil"]
_DATASETNAME = "fakenews_ph"
_DESCRIPTION = """\
Fake news articles were sourced from online sites that were tagged as fake
news sites by the non-profit independent media fact-checking organization
Verafiles and the National Union of Journalists in the Philippines (NUJP).
Real news articles were sourced from mainstream news websites in the
Philippines, including Pilipino Star Ngayon, Abante, and Bandera.
"""

_HOMEPAGE = "https://github.com/jcblaisecruz02/Tagalog-fake-news"
_LICENSE = Licenses.GPL_3_0.value
_URL = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/fakenews/fakenews.zip"

_SUPPORTED_TASKS = [Tasks.HOAX_NEWS_CLASSIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class FakeNewsFilipinoDataset(datasets.GeneratorBasedBuilder):
    """Fake News Filipino Dataset from https://huggingface.co/datasets/fake_news_filipino"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "text"
    LABEL_CLASSES = ["0", "1"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "article": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=self.LABEL_CLASSES),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Return SplitGenerators."""
        data_dir = Path(dl_manager.download_and_extract(_URL))
        train_path = data_dir / "fakenews" / "full.csv"
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_path, "split": "train"},
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file,
                quotechar='"',
                delimiter=",",
                quoting=csv.QUOTE_ALL,
                skipinitialspace=True,
            )
            next(csv_reader)
            for id_, row in enumerate(csv_reader):
                label, article = row
                if self.config.schema == "source":
                    yield id_, {"label": label, "article": article}
                if self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    yield id_, {"id": id_, "label": label, "text": article}



# coding=utf-8

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# no bibtex citation
_CITATION = ""

_DATASETNAME = "kheng_info"

_DESCRIPTION = """\
The Kheng.info Speech dataset was derived from recordings of Khmer words on the Khmer dictionary website kheng.info.
The recordings were recorded by a native Khmer speaker.
The recordings are short, generally ranging between 1 to 2 seconds only.
"""

_HOMEPAGE = "https://huggingface.co/datasets/seanghay/khmer_kheng_info_speech"

_LANGUAGES = ["khm"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://huggingface.co/datasets/seanghay/khmer_kheng_info_speech/resolve/main/data/train-00000-of-00001-4e7ad082a34164d1.parquet",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class KhengInfoDataset(datasets.GeneratorBasedBuilder):
    """This is the Kheng.info Speech dataset, which wasderived from recordings on the Khmer dictionary website kheng.info"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_sptext",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_sptext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"word": datasets.Value("string"), "duration_ms": datasets.Value("int64"), "audio": datasets.Audio(sampling_rate=16_000)})

        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                },
            )
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        df = pd.read_parquet(filepath, engine="pyarrow")
        if self.config.schema == "source":
            for _id, row in df.iterrows():
                yield _id, {"word": row["word"], "duration_ms": row["duration_ms"], "audio": row["audio"]}
        elif self.config.schema == "seacrowd_sptext":
            for _id, row in df.iterrows():
                yield _id, {
                    "id": _id,
                    "path": row["audio"],
                    "audio": row["audio"],
                    "text": row["word"],
                    "speaker_id": None,
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": None,
                    },
                }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{hlaing-2020-myanmar,
  author={Hlaing, Zar Zar and Thu, Ye Kyaw and Wai, Myat Myo Nwe and Supnithi, Thepchai and Netisopakul, Ponrudee},
  booktitle={2020 15th International Joint Symposium on Artificial Intelligence and Natural Language Processing (iSAI-NLP)},
  title={Myanmar POS Resource Extension Effects on Automatic Tagging Methods},
  year={2020},
  pages={1-6},
  doi={10.1109/iSAI-NLP51646.2020.9376835}}
@inproceedings{htike2017comparison,
  title={Comparison of six POS tagging methods on 10K sentences Myanmar language (Burmese) POS tagged corpus},
  author={Htike, Khin War War and Thu, Ye Kyaw and Zuping Zhang, Win Pa Pa and Sagisaka, Yoshinori and Iwahashi, Naoto},
  booktitle={Proceedings of the CICLING},
  year={2017}
}
"""

_LOCAL = False
_LANGUAGES = ["mya"]
_DATASETNAME = "mypos"
_DESCRIPTION = """\
This version of the myPOS corpus extends the original myPOS corpus from
11,000 to 43,196 Burmese sentences by adding data from the ASEAN MT NECTEC
corpus and two developed parallel corpora (Myanmar-Chinese and
Myanmar-Korean). The original 11,000 sentences were collected from Wikipedia
and includes various topics such as economics, history, news, politics and
philosophy. The format used in the corpus is word/POS-tag, and the pipe
delimiter "
"""

_HOMEPAGE = "https://github.com/ye-kyaw-thu/myPOS"
_LICENSE = Licenses.CC_BY_NC_SA_4_0.value
_URL = "https://raw.githubusercontent.com/ye-kyaw-thu/myPOS/master/corpus-ver-3.0/corpus/mypos-ver.3.0.txt"

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]
_SOURCE_VERSION = "3.0.0"
_SEACROWD_VERSION = "1.0.0"


class MyPOSDataset(datasets.GeneratorBasedBuilder):
    """MyPOS dataset from https://github.com/ye-kyaw-thu/myPOS"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "seq_label"
    # Reference: https://github.com/ye-kyaw-thu/myPOS/tree/master#pos-tags
    LABEL_CLASSES = ["abb", "adj", "adv", "conj", "fw", "int", "n", "num", "part", "ppm", "pron", "punc", "sb", "tn", "v"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        # No specific schema from source, we will just reuse the seacrowd schema
        features = schemas.seq_label_features(self.LABEL_CLASSES)
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_file = Path(dl_manager.download_and_extract(_URL))
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_file})]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""

        with open(filepath, encoding="utf-8") as f:
            lines = f.readlines()

        for idx, line in enumerate(lines):

            line = line.rstrip("\n")
            tags = self._tokenize(line)

            split_token = [tag.split("/") for tag in tags if tag]
            tokens = [split[0] for split in split_token]
            labels = [split[1] for split in split_token]
            example = {"id": str(idx), "tokens": tokens, "labels": labels}

            yield idx, example

    def _tokenize(self, sentence: str) -> List[str]:
        """Tokenize Myanmar text

        From the README: https://github.com/ye-kyaw-thu/myPOS/tree/master#word-segmentation
        Important things to point out:
        - Words composed of single or multiple syllables are usually not separated by white space.
        - There are no clear rules for using spaces in Myanmar language.
        - The authors used six rules for word segmentation
        """
        final_tokens = []

        # Segment via spaces (c.f. "Spaces are used for easier reading and generally put between phrases")
        init_tokens = sentence.split(" ")
        # Segment breakpoints ('|' pipe character) for compount words
        for token in init_tokens:
            final_tokens.extend(token.split("|"))

        return final_tokens



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
Wannaphong Phatthiyaphaibun. (2021). PyThaiNLP/Thai-Lao-Parallel-Corpus: \
Thai Lao Parallel corpus v0.7 (v0.7). Zenodo \
https://doi.org/10.5281/zenodo.5807093"""

_DATASETNAME = "tha_lao_embassy_parcor"

_DESCRIPTION = """\
Thai-Lao Parallel Corpus contains equivalent Thai and Lao sentence pairs \
derived from the website of the Royal Thai Embassy in Vientiane, Laos.
"""

_HOMEPAGE = "https://github.com/PyThaiNLP/Thai-Lao-Parallel-Corpus/tree/master"
_LANGUAGES = ["tha", "lao"]
_LICENSE = Licenses.CC0_1_0.value

_LOCAL = False
_URLS = {_DATASETNAME: "https://github.com/PyThaiNLP/Thai-Lao-Parallel-Corpus/raw/master/vientiane-thaiembassy-sent.csv"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "0.7.0"
_SEACROWD_VERSION = "1.0.0"


class ThaLaoEmbassyParcorDataset(datasets.GeneratorBasedBuilder):
    """Thai-Lao Parallel Corpus contains equivalent Thai and Lao sentence pairs \
    derived from the website of the Royal Thai Embassy in Vientiane, Laos."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "lao_sent": datasets.Value("string"),
                    "thai_sent": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        filename = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(filename),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        dataset = pd.read_csv(filepath)

        if self.config.schema == "source":
            for i, row in dataset.iterrows():
                yield i, {"lao_sent": row["lao_sent"], "thai_sent": row["thai_sent"]}

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in dataset.iterrows():
                yield i, {
                    "id": i,
                    "text_1": row["lao_sent"],
                    "text_2": row["thai_sent"],
                    "text_1_name": "lao",
                    "text_2_name": "tha",
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{nguyen2018vlsp,
  title={VLSP shared task: sentiment analysis},
  author={Nguyen, Huyen TM and Nguyen, Hung V and Ngo, \
Quyen T and Vu, Luong X and Tran, Vu Mai and Ngo, Bach X and Le, Cuong A},
  journal={Journal of Computer Science and Cybernetics},
  volume={34},
  number={4},
  pages={295--310},
  year={2018}
}
"""
_DATASETNAME = "vlsp2016_sa"

_DESCRIPTION = """\
The SA-VLSP2016 dataset were collected from three source sites which are tinhte.vn, \
vnexpress.net and Facebook, and used for the sentiment analysis task. The data consists \
of comments of technical articles on those sites. Each comment is given one of \
four labels: POS (positive), NEG (negative), NEU (neutral) and USELESS (filter-out).
"""

_HOMEPAGE = "https://vlsp.org.vn/resources-vlsp2016"
_LANGUAGES = ["vie"]

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value
_LOCAL = True

_URLS = {}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"
_TAGS = ["POS", "NEG", "NEU"]


class VLSP2016SADataset(datasets.GeneratorBasedBuilder):
    """The SA-VLSP2016 dataset, used for sentiment analysis, comprises comments from technical \
    articles on tinhte.vn, vnexpress.net, and Facebook, each labeled as positive, negative, neutral, or filter-out."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_tokenized_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}_tokenized",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=_TAGS),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(_TAGS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "SA2016-training_data"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "SA2016-TestData-Ans"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if split == "dev":
            if self.config.schema in ["source", f"seacrowd_{self.SEACROWD_SCHEMA_NAME}"]:
                labelfile = "test_raw_ANS.txt"
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}_tokenized":
                labelfile = "test_tokenized_ANS.txt"

            with open(os.path.join(filepath, labelfile)) as file:
                data = file.read()

            pattern = re.compile("(?P<sentence>.+)\n(?P<label>(POS|NEG|NEU))\n")

            if self.config.schema in ["source", f"seacrowd_{self.SEACROWD_SCHEMA_NAME}", f"seacrowd_{self.SEACROWD_SCHEMA_NAME}_tokenized"]:
                for i, match in enumerate(pattern.finditer(data)):
                    yield i, {"id": i, "text": match.group("sentence").replace("\xa0", " "), "label": match.group("label")}

        else:
            labeltext = {"POS": [], "NEG": [], "NEU": []}
            if self.config.schema in ["source", f"seacrowd_{self.SEACROWD_SCHEMA_NAME}"]:
                positive = "SA-training_positive.txt"
                negative = "SA-training_negative.txt"
                neutral = "SA-training_neutral.txt"
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}_tokenized":
                positive = "train_positive_tokenized.txt"
                negative = "train_negative_tokenized.txt"
                neutral = "train_neutral_tokenized.txt"

            for labelsplit, labelfile in zip(labeltext.keys(), [positive, negative, neutral]):
                with open(os.path.join(filepath, labelfile)) as file:
                    data = file.read()
                labeltext[labelsplit] = data.split("\n\n")[:-1]

            if self.config.schema in ["source", f"seacrowd_{self.SEACROWD_SCHEMA_NAME}", f"seacrowd_{self.SEACROWD_SCHEMA_NAME}_tokenized"]:
                idcounter = 0
                for label, sentences in labeltext.items():
                    for sentence in sentences:
                        yield idcounter, {"id": idcounter, "text": sentence, "label": label}
                        idcounter = idcounter + 1



from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{koto-koto-2020-towards,
    title = "Towards Computational Linguistics in {M}inangkabau Language: Studies on Sentiment Analysis and Machine Translation",
    author = "Koto, Fajri  and
      Koto, Ikhwan",
    booktitle = "Proceedings of the 34th Pacific Asia Conference on Language, Information and Computation",
    month = oct,
    year = "2020",
    address = "Hanoi, Vietnam",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2020.paclic-1.17",
    pages = "138--148",
}
"""

_LOCAL = False
_LANGUAGES = ["min", "ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "minangnlp_mt"
_DESCRIPTION = """\
In this work, we create Minangkabau–Indonesian (MIN-ID) parallel corpus by using Wikipedia. We obtain 224,180 Minangkabau and
510,258 Indonesian articles, and align documents through title matching, resulting in 111,430 MINID document pairs.
After that, we do sentence segmentation based on simple punctuation heuristics and obtain 4,323,315 Minangkabau sentences. We
then use the bilingual dictionary to translate Minangkabau article (MIN) into Indonesian language (ID'). Sentence alignment is conducted using
ROUGE-1 (F1) score (unigram overlap) (Lin, 2004) between ID’ and ID, and we pair each MIN sentencewith an ID sentence based on the highest ROUGE1.
We then discard sentence pairs with a score of less than 0.5 to result in 345,146 MIN-ID parallel sentences.
We observe that the sentence pattern in the collection is highly repetitive (e.g. 100k sentences are about biological term definition). Therefore,
we conduct final filtering based on top-1000 trigram by iteratively discarding sentences until the frequency of each trigram equals to 100. Finally, we
obtain 16,371 MIN-ID parallel sentences and conducted manual evaluation by asking two native Minangkabau speakers to assess the adequacy and
fluency (Koehn and Monz, 2006). The human judgement is based on scale 1–5 (1 means poor quality and 5 otherwise) and conducted against 100 random
samples. We average the weights of two annotators before computing the overall score, and we achieve 4.98 and 4.87 for adequacy and fluency respectively.
This indicates that the resulting corpus is high-quality for machine translation training.
"""

_HOMEPAGE = "https://github.com/fajri91/minangNLP"
_LICENSE = "MIT"
_URLS = {
    _DATASETNAME: "https://github.com/fajri91/minangNLP/archive/refs/heads/master.zip",
}
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
# Dataset does not have versioning
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class MinangNLPmt(datasets.GeneratorBasedBuilder):
    """16,371-size parallel Minangkabau-Indonesian sentence pairs."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="minangnlp_mt_source",
            version=SOURCE_VERSION,
            description="MinangNLP Machine Translation source schema",
            schema="source",
            subset_id="minangnlp_mt",
        ),
        SEACrowdConfig(
            name="minangnlp_mt_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="MinangNLP Machine Translation Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="minangnlp_mt",
        ),
    ]

    DEFAULT_CONFIG_NAME = "minangnlp_mt_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src": datasets.Value("string"),
                    "tgt": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = Path(dl_manager.download_and_extract(urls)) / "minangNLP-master" / "translation" / "wiki_data"
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "src_filepath": data_dir / "src_train.txt",
                    "tgt_filepath": data_dir / "tgt_train.txt",
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "src_filepath": data_dir / "src_test.txt",
                    "tgt_filepath": data_dir / "tgt_test.txt",
                    "split": "test",
                },
            ),
            # Dataset has a secondary test split
            # datasets.SplitGenerator(
            #     name=datasets.Split.TEST,
            #     gen_kwargs={
            #         "src_filepath": data_dir / "src_test_sent.txt",
            #         "tgt_filepath": data_dir / "tgt_test_sent.txt",
            #         "split": "test_sent",
            #     },
            # ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "src_filepath": data_dir / "src_dev.txt",
                    "tgt_filepath": data_dir / "tgt_dev.txt",
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, src_filepath: Path, tgt_filepath: Path, split: str) -> Tuple[int, Dict]:
        with open(src_filepath, encoding="utf-8") as fsrc, open(tgt_filepath, encoding="utf-8") as ftgt:
            for idx, pair in enumerate(zip(fsrc, ftgt)):
                src, tgt = pair
                if self.config.schema == "source":
                    row = {
                        "id": str(idx),
                        "src": src,
                        "tgt": tgt,
                    }
                    yield idx, row

                elif self.config.schema == "seacrowd_t2t":
                    row = {
                        "id": str(idx),
                        "text_1": src,
                        "text_2": tgt,
                        "text_1_name": "min",
                        "text_2_name": "id",
                    }
                    yield idx, row



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.sea_datasets.id_short_answer_grading.utils.id_short_answer_grading_utils import \
    create_saintek_and_soshum_dataset
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article{
    JLK,
    author = {Muh Haidir and Ayu Purwarianti},
    title = { Short Answer Grading Using Contextual Word Embedding and Linear Regression},
    journal = {Jurnal Linguistik Komputasional},
    volume = {3},
    number = {2},
    year = {2020},
    keywords = {},
    abstract = {Abstract—One of the obstacles in an efficient MOOC is the evaluation of student answers, including the short answer grading which requires large effort from instructors to conduct it manually.
                Thus, NLP research in short answer grading has been conducted in order to support the automation, using several techniques such as rule
                and machine learning based. Here, we’ve conducted experiments on deep learning based short answer grading to compare the answer
                representation and answer assessment method. In the answer representation, we compared word embedding and sentence embedding models
                such as BERT, and its modification. In the answer assessment method, we use linear regression. There are 2 datasets that we used, available
                English short answer grading dataset with 80 questions and 2442 to get the best configuration for model and Indonesian short answer grading
                dataset with 36 questions and 9165 short answers as testing data. Here, we’ve collected Indonesian short answers for Biology and Geography
                subjects from 534 respondents where the answer grading was done by 7 experts. The best root mean squared error for both dataset was achieved
                by using BERT pretrained, 0.880 for English dataset dan 1.893 for Indonesian dataset.},
    issn = {2621-9336},	pages = {54--61},	doi = {10.26418/jlk.v3i2.38},
    url = {https://inacl.id/journal/index.php/jlk/article/view/38}
}\
"""
_DATASETNAME = "id_short_answer_grading"

_DESCRIPTION = """\
Indonesian short answers for Biology and Geography subjects from 534 respondents where the answer grading was done by 7 experts.\
"""

_HOMEPAGE = "https://github.com/AgeMagi/tugas-akhir"
_LOCAL = False
_LANGUAGES = ["ind"]

_LICENSE = "Unknown"

_URLS = {
    "saintek": {
        "train": {
            "question": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/question-saintek.csv",
            "score": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/score-saintek.csv",
        },
        "test": {
            "question": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/question-saintek-test.csv",
            "score": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/score-saintek-test.csv",
        },
    },
    "soshum": {
        "train": {
            "question": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/question-soshum.csv",
            "score": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/score-soshum.csv",
        },
        "test": {
            "question": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/question-soshum-test.csv",
            "score": "https://raw.githubusercontent.com/AgeMagi/tugas-akhir/master/data/score-soshum-test.csv",
        },
    },
}

_SUPPORTED_TASKS = [Tasks.SHORT_ANSWER_GRADING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IdShortAnswerGrading(datasets.GeneratorBasedBuilder):
    """Indonesian short answers for Biology and Geography subjects from 534 respondents where the answer grading was done by 7 experts."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_short_answer_grading_source",
            version=SOURCE_VERSION,
            description="id_short_answer_grading source schema",
            schema="source",
            subset_id="id_short_answer_grading",
        ),
        SEACrowdConfig(
            name="id_short_answer_grading_seacrowd_pairs_score",
            version=SEACROWD_VERSION,
            description="id_short_answer_grading Nusantara schema",
            schema="seacrowd_pairs_score",
            subset_id="id_short_answer_grading",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_short_answer_grading_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("int64"),
                    "type-problem": datasets.Value("int64"),
                    "pertanyaan": datasets.Value("string"),
                    "kunci-jawaban": datasets.Value("string"),
                    "jawaban": datasets.Value("string"),
                    "score": datasets.Value("int64"),
                }
            )
        elif self.config.schema == "seacrowd_pairs_score":
            features = schemas.pairs_features([0, 1, 2, 3, 4, 5])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        saintek_question = Path(dl_manager.download_and_extract(_URLS["saintek"]["train"]["question"]))
        saintek_score = Path(dl_manager.download_and_extract(_URLS["saintek"]["train"]["score"]))
        saintek_question_test = Path(dl_manager.download_and_extract(_URLS["saintek"]["test"]["question"]))
        saintek_score_test = Path(dl_manager.download_and_extract(_URLS["saintek"]["test"]["score"]))

        soshum_question = Path(dl_manager.download_and_extract(_URLS["soshum"]["train"]["question"]))
        soshum_score = Path(dl_manager.download_and_extract(_URLS["soshum"]["train"]["score"]))
        soshum_question_test = Path(dl_manager.download_and_extract(_URLS["soshum"]["test"]["question"]))
        soshum_score_test = Path(dl_manager.download_and_extract(_URLS["soshum"]["test"]["score"]))

        data_files = {
            "saintek_question": saintek_question,
            "saintek_score": saintek_score,
            "saintek_question_test": saintek_question_test,
            "saintek_score_test": saintek_score_test,
            "soshum_question": soshum_question,
            "soshum_score": soshum_score,
            "soshum_question_test": soshum_question_test,
            "soshum_score_test": soshum_score_test,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "saintek_question": os.path.join(data_files["saintek_question"]),
                    "soshum_question": os.path.join(data_files["soshum_question"]),
                    "saintek_score": os.path.join(data_files["saintek_score"]),
                    "soshum_score": os.path.join(data_files["soshum_score"]),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "saintek_question": os.path.join(data_files["saintek_question_test"]),
                    "soshum_question": os.path.join(data_files["soshum_question_test"]),
                    "saintek_score": os.path.join(data_files["saintek_score_test"]),
                    "soshum_score": os.path.join(data_files["soshum_score_test"]),
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, saintek_question: Path, soshum_question: Path, saintek_score: Path, soshum_score: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = create_saintek_and_soshum_dataset(saintek_question, soshum_question, saintek_score, soshum_score)
        if self.config.schema == "source":
            for row in df.itertuples():
                entry = {
                    "index": row.index,
                    "type-problem": row.type_problem,
                    "pertanyaan": row.pertanyaan,
                    "kunci-jawaban": row.kunci_jawaban,
                    "jawaban": row.jawaban,
                    "score": row.score,
                }
                yield row.index, entry

        elif self.config.schema == "seacrowd_pairs_score":
            for row in df.itertuples():
                entry = {
                    "id": str(row.index),
                    "text_1": row.pertanyaan,
                    "text_2": row.jawaban,
                    "label": row.score,
                }
                yield row.index, entry



from pathlib import Path
from typing import List

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas

import pandas as pd

_CITATION = """\
    @misc{wibisono2022indotacos,
        title = {IndoTacos},
        howpublished = {\\url{https://www.kaggle.com/datasets/christianwbsn/indonesia-tax-court-verdict}},
        note = {Accessed: 2022-09-22}
    }
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indotacos"

_DESCRIPTION = """\
Predicting the outcome or the probability of winning a legal case has always been highly attractive in legal sciences and practice.
Hardly any dataset has been developed to analyze and accelerate the research of court verdict analysis.
Find out what factor affects the outcome of tax court verdict using Natural Language Processing.
"""

_HOMEPAGE = "https://www.kaggle.com/datasets/christianwbsn/indonesia-tax-court-verdict"

_LICENSE = "Creative Common Attribution Share-Alike 4.0 International"

# For publicly available datasets you will most likely end up passing these URLs to dl_manager in _split_generators.
# In most cases the URLs will be the same for the source and seacrowd config.
# However, if you need to access different files for each config you can have multiple entries in this dict.
# This can be an arbitrarily nested dict/list of URLs (see below in `_split_generators` method)
_URLS = {_DATASETNAME: {"indotacos": "https://huggingface.co/datasets/christianwbsn/indotacos/resolve/main/indonesia_tax_court_verdict.csv"}}

_SUPPORTED_TASKS = [Tasks.TAX_COURT_VERDICT]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IndoTacos(datasets.GeneratorBasedBuilder):
    """IndoTacos, an Indonesian Tax Court verdict summary containing 12283 tax court cases provided by perpajakan.ddtc.co.id."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indotacos_source",
            version=SOURCE_VERSION,
            description="indotacos source schema",
            schema="source",
            subset_id="indotacos",
        ),
        SEACrowdConfig(
            name="indotacos_seacrowd_text",
            version=SEACROWD_VERSION,
            description="IndoTacos Nusantara schema",
            schema="seacrowd_text",
            subset_id="indotacos",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indotacos_source"
    labels = ["mengabulkan sebagian", "mengabulkan seluruhnya", "menolak", "lain-lain", "menambah pajak", "mengabulkan", "membetulkan"]

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "text": datasets.Value("string"),
                    "nomor_putusan": datasets.Value("string"),
                    "tahun_pajak": datasets.Value("int32"),
                    "jenis_pajak": datasets.Value("string"),
                    "tahun_putusan": datasets.Value("int32"),
                    "pokok_sengketa": datasets.Value("string"),
                    "jenis_putusan": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(self.labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        url = _URLS["indotacos"]
        path = dl_manager.download(url)["indotacos"]
        data_files = {"train": path}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                },
            )
        ]

    def _generate_examples(self, filepath: Path):
        df = pd.read_csv(filepath)
        if self.config.schema == "source":
            row_id = 1
            for row in df.itertuples():
                ex = {
                    "id": str(row_id),
                    "text": row.text,
                    "nomor_putusan": row.nomor_putusan,
                    "tahun_pajak": row.tahun_pajak,
                    "jenis_pajak": row.jenis_pajak,
                    "tahun_putusan": row.tahun_putusan,
                    "pokok_sengketa": row.pokok_sengketa,
                    "jenis_putusan": row.jenis_putusan,
                }
                yield row_id, ex
                row_id += 1
        elif self.config.schema == "seacrowd_text":
            row_id = 1
            for row in df.itertuples():
                ex = {
                    "id": str(row_id),
                    "text": {"text": row.text, "nomor_putusan": row.nomor_putusan, "tahun_pajak": row.tahun_pajak, "jenis_pajak": row.jenis_pajak, "tahun_putusan": row.tahun_putusan, "pokok_sengketa": row.pokok_sengketa},
                    "label": row.jenis_putusan,
                }
                yield row_id, ex
                row_id += 1
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple
from zipfile import ZipFile

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME

_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME
_LOCAL = False

_CITATION = """\
@article{nllb2022,
  author    = {NLLB Team, Marta R. Costa-jussà, James Cross, Onur Çelebi, Maha Elbayad, Kenneth Heafield, Kevin Heffernan, Elahe Kalbassi,  Janice Lam, Daniel Licht, Jean Maillard, Anna Sun, Skyler Wang, Guillaume Wenzek, Al Youngblood, Bapi Akula, Loic Barrault, Gabriel Mejia Gonzalez, Prangthip Hansanti, John Hoffman, Semarley Jarrett, Kaushik Ram Sadagopan, Dirk Rowe, Shannon Spruit, Chau Tran, Pierre Andrews, Necip Fazil Ayan, Shruti Bhosale, Sergey Edunov, Angela Fan, Cynthia Gao, Vedanuj Goswami, Francisco Guzmán, Philipp Koehn, Alexandre Mourachko, Christophe Ropers, Safiyyah Saleem, Holger Schwenk, Jeff Wang},
  title     = {No Language Left Behind: Scaling Human-Centered Machine Translation},
  year      = {2022}
}
"""

_DATASETNAME = "toxicity_200"

_DESCRIPTION = """\
Toxicity-200 is a wordlist to detect toxicity in 200 languages. It contains files that include frequent words and phrases generally considered toxic because they represent: 1) frequently used profanities; 2) frequently used insults and hate speech terms, or language used to bully, denigrate, or demean; 3) pornographic terms; and 4) terms for body parts associated with sexual activity.
"""

_HOMEPAGE = "https://github.com/facebookresearch/flores/blob/main/toxicity"

_LICENSE = "CC-BY-SA 4.0"
_LANGUAGES = ["ind", "ace", "bjn", "bug", "jav"]
_LANGUAGE_MAP = {"ind": "Indonesia", "ace": "Aceh", "bjn": "Banjar", "bug": "Bugis", "jav": "Java"}
_URLS = {
    "toxicity_200": "https://tinyurl.com/NLLB200TWL",
}
_PASS = "tL4nLLb"
_SUPPORTED_TASKS = []  # [Tasks.SELF_SUPERVISED_PRETRAINING]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(lang, schema, version):
    if lang == "":
        raise ValueError(f"Invalid lang {lang}")

    if schema != "source":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="toxicity_200_{lang}_{schema}".format(lang=lang, schema=schema),
        version=datasets.Version(version),
        description="toxicity 200 with {schema} schema for {lang} language".format(lang=_LANGUAGE_MAP[lang], schema=schema),
        schema=schema,
        subset_id="toxicity_200",
    )

def extract_toxic_zip(filepath):
    with ZipFile(filepath, "r") as zip:
        zip.extractall(path=filepath[:-4], pwd=_PASS.encode("utf-8"))
    return filepath[:-4]

class Toxicity200(datasets.GeneratorBasedBuilder):
    """TODO: Short description of my dataset."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in _LANGUAGES]

    DEFAULT_CONFIG_NAME = "toxicity_200_ind_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "toxic_word": [datasets.Value("string")]})
        else:
            raise NotImplementedError()
        
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = Path(dl_manager.download_and_extract(urls)) / "NLLB-200_TWL"

        data_subdir = {
            "ind": os.path.join(data_dir, "ind_Latn_twl.zip"),
            "ace": os.path.join(data_dir, "ace_Latn_twl.zip"),
            "bjn": os.path.join(data_dir, "bjn_Latn_twl.zip"),
            "bug": os.path.join(data_dir, "bug_Latn_twl.zip"),
            "jav": os.path.join(data_dir, "jav_Latn_twl.zip"),
        }

        lang = self.config.name.split("_")[2]
        text_dir = extract_toxic_zip(data_subdir[lang])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {"text_file": os.path.join(text_dir, lang + "_Latn_twl.txt")},
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        text = open(filepath["text_file"], "r").readlines()
        word_list = list(map(str.strip, text))
        print(text[:5])
        if self.config.schema == "source":
            for id, word in enumerate(word_list):
                row = {"id": str(id), "toxic_word": [word]}
                yield id, row
        else:
            raise ValueError(f"Invalid config: {self.config.name}")

if __name__ == "__main__":
    datasets.load_dataset(__file__)
